// This model version lets you specify separate priors for the air temperature
// In this model, air temperature affects variance and mean

functions {
  // Must do this in a separate function so that every value of sigma isn't returned in model
  vector getSigma(matrix x, vector gamma, real gamma0, int N, vector var_supp) {
    vector[N] sigma;
    sigma = x * gamma + gamma0 + var_supp;
    if (min(sigma) < 0)
      reject("sigma below 0!");
    return sqrt(sigma);
  }
}

data {
  int<lower=0> N; // number of observations
  int<lower=0> K; // number of predictors
  matrix[N, K] x; // predictor matrix (scaled air temp should be last col)
  vector[N] y;    // outcome vector
  int l_cens;    // number of left-censored points
  int r_cens;    // number of right-censored points
  int n_cens;    // number of non-censored points
  vector[N] var_supp; // supplemental variance from sampling error
  real L; // left (lower) censoring limit
  real U; // right (upper) censoring limit
  
  real ALPHA_MU;
  real ALPHA_SD;
  real BETA_SD;
  real GAMMA_SD;
  real GAMMA0_MU;
  real GAMMA0_SD;
  real TEMP_MU_MU;
  real TEMP_MU_SD;
  real TEMP_SIGMA_MU;
  real TEMP_SIGMA_SIGMA;
  
  int DO_FIT;
  int DO_SIM;
}

// Here we handle data censoring by counting and removing censored points
transformed data {
  int N_obs; // number of non-censored points
  int N_l_cens; // number of left censored points
  int N_r_cens; // number of right censored points
  vector[n_cens] y_obs; // observed non-censored values of y
  matrix[n_cens, K] x_obs; // observed non-censored values of x
  matrix[l_cens, K] x_l_cens; // predictors for left censored points
  matrix[r_cens, K] x_r_cens; // predictors for right censored points
  vector[n_cens] var_supp_obs; // predictors for left censored points
  vector[l_cens] var_supp_l_cens; // predictors for left censored points
  vector[r_cens] var_supp_r_cens; // predictors for right censored points

  N_l_cens = 0;
  N_r_cens = 0;
  N_obs = 0;
  
  for (n in 1:N) {
    if (y[n] <= L) {
      N_l_cens = N_l_cens + 1;  // increment counter first to make 1-based indexing work
      x_l_cens[N_l_cens] = x[n];
      var_supp_l_cens[N_l_cens] = var_supp[n];
      }
    else if (y[n] >= U) {
      N_r_cens = N_r_cens + 1;
      x_r_cens[N_r_cens] = x[n];
      var_supp_r_cens[N_r_cens] = var_supp[n];
      }
    else {
      N_obs = N_obs + 1;
      y_obs[N_obs] = y[n];
      x_obs[N_obs] = x[n];
      var_supp_obs[N_obs] = var_supp[n];
      }
  }
  print("Number of left-censored points = ", N_l_cens)
  print("Number of right-censored points = ", N_r_cens)
  print("Number of non-censored points = ", N_obs)
}

parameters {
  real<lower=0,upper=1> alpha;                 // mean intercept
  real<lower=0> gamma0;       // var intercept
  vector[K-1] beta_partial;   // mean coefficients for predictors
  vector[K-1] gamma_partial;  // var coefficients for predictors (minus air temp)
  real tau_mu;                // mean coefficient for air temperature
  real tau_sigma;             // var coefficient for air temperature
}

transformed parameters {
  vector[K] beta;  // mean coefficients for predictors w/temp added
  vector[K] gamma;  // mean coefficients for predictors w/temp added
  // there may be a more efficient way to concatenate, but I don't know it
  for (k in 1:(K-1)) {
    beta[k] = beta_partial[k]; 
    gamma[k] = gamma_partial[k];
  }
  beta[K] = tau_mu;
  gamma[K] = tau_sigma;
}

model {
    // Priors
    alpha ~ normal(ALPHA_MU, ALPHA_SD);
    beta_partial ~ normal(0, BETA_SD);
    gamma_partial ~ normal(0, GAMMA_SD);
    gamma0 ~ normal(GAMMA0_MU, GAMMA0_SD);
    tau_mu ~ normal(TEMP_MU_MU, TEMP_MU_SD);
    tau_sigma ~ normal(TEMP_SIGMA_MU, TEMP_SIGMA_SIGMA);
    
    if(DO_FIT == 1) {

      // Likelihood for non-censored data
      y_obs ~ normal(alpha + x_obs * beta, getSigma(x_obs, gamma, gamma0, N_obs, var_supp_obs));
      
      // Censored data
      if (N_l_cens > 0) {
        for (n in 1:N_l_cens)
          target += normal_lcdf(L | alpha + x_l_cens[n] * beta, sqrt(gamma0 + x_l_cens[n] * gamma + var_supp_l_cens[n]));
      }
      if (N_r_cens > 0) {
        for (n in 1:N_r_cens)
          target += normal_lccdf(U | alpha + x_r_cens[n] * beta, sqrt(gamma0 + x_r_cens[n] * gamma + var_supp_r_cens[n]));
      }
      
    } //else {
        //if (min(x * gamma + gamma0 + var_supp) < 0)
          //reject("sigma below 0!");
          //}
    
}

generated quantities {
  if(DO_SIM == 1) {
    //vector[N] log_lik;
    vector[N] y_rep;
    for (n in 1:N) {
      //log_lik[n] = normal_lpdf(y[n] | alpha + x[n] * beta, sqrt(gamma0 + x[n] * gamma + var_supp[n]));
      y_rep[n] = normal_rng(alpha + x[n] * beta, sqrt(gamma0 + x[n] * gamma + var_supp[n]));
      if (y_rep[n] > U)
        y_rep[n] = U;
      if (y_rep[n] < L)
        y_rep[n] = L;
    }
  }
}
